/****************************************************************************
 METEOR

 Este programa muestra el manejo de algunas funciones graficas. La velocidad
 de ejecucion depende directamente de la velocidad del microprocesador, ya
 que se controla mediante un contador. Esta hecho en Borland C 3.1, pero creo
 que correr en cualquier versin posterior.

 Programado por Jorge Ramrez Flores, de la Escuela Superior de Computo del
 IPN, 1er Semestre. Correo: jorrami@fastnet.com.mx. Si encuentras un bug o
 tienes una mejora, no dudes en escribirme.
****************************************************************************/

#include <dos.h>
#include <math.h>
#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <graphics.h>

// Libreria de control de mouse
#include "mouse.h"

// Definicion de macros
#define maxx		getmaxx ()
#define maxy		getmaxy ()
#define medx		maxx / 2
#define medy		maxy / 2

// Definicion de constantes
#define disty		450
#define ancho		20
#define ancho_e		10
#define maxenemigos	49
#define margen		20

// Estructura con los datos de los enemigos
struct ENEMIGO
	{int x, y, visible, enmovimiento;
	} enemigo [maxenemigos + 1];

// Estructura con los datos del enemigo
struct BALA
	{int x, y1, y2, visible;
	} bala;

// Prototipos de las funciones
void inicializar ();
void escribe (int x, int y, char  *cadena, ...);
void dibujar ();
void mover_n (int x);
void crear_enemigos ();
void mover_e ();
void disparar ();
void avanzar ();
void barra_estado ();
void estallido_e ();
void estallido_n ();
void terminar ();
int juego ();
void subir_nivel ();

// Variables globales del juego
int navex, cont = 0, retardo = 150, retardo_tiro = 15, vidas = 5, destruidos = 0, nivel = 1, maxmov = 5;
void *imagen_n, *imagen_e, *explosion;

// Programa principal
void main ()
{int terminar = 0;
 inicializar (),
 dibujar ();
 do
   terminar = juego ();
 while  (!terminar);
 free (imagen_n),
 free (imagen_e),
 closegraph (),
 exit (0);
}

// Inicializa graficos
void inicializar ()
  {int a=0, b;
   initgraph (&a, &b, ""),
   navex = medx,
   bala.x = navex + ancho / 2,
   bala.y1 = disty - 6,
   bala.y2 = disty - 1,
   bala.visible = 0;
  }

// Escribe una cadena con formato en x,y
void escribe (int x, int y, char  *cadena, ...)
  {va_list  argptr;
   char str [140];
   va_start (argptr, cadena);
   vsprintf (str, cadena, argptr);
   outtextxy (x, y, str);
   va_end (argptr);
  }

// Traza y almacena las imagenes de la nave, enemigos y explosiones
void dibujar ()
  {int coords [] = {ancho/2,0, 0,ancho/2, ancho,ancho/2}, color=getcolor ();
   struct fillsettingstype tipo;
   getfillsettings (&tipo),
   setcolor (1), 		   // Nave
   setfillstyle (1, 1),
   fillpoly (3, coords),
   imagen_n = malloc (imagesize (0,0, ancho,ancho/2)),
   getimage (0,0, ancho,ancho/2, imagen_n),
   putimage (0,0, imagen_n, XOR_PUT),
   setcolor (3),		   // Enemigo
   setfillstyle (1, 11),
   fillellipse (ancho_e/2,ancho_e/2, ancho_e/2,ancho_e/2),
   imagen_e = malloc (imagesize (0,0, ancho_e,ancho_e)),
   getimage (0,0, ancho_e,ancho_e, imagen_e),
   putimage (0,0, imagen_e, XOR_PUT),
   setcolor (14),		   // Explosion
   setfillstyle (1, 14),
   fillellipse (ancho_e/2,ancho_e/2, ancho_e/2,ancho_e/2),
   explosion = malloc (imagesize (0,0, ancho_e,ancho_e)),
   getimage (0,0, ancho_e,ancho_e, explosion),
   putimage (0,0, explosion, XOR_PUT),
   setcolor (color),
   setfillstyle (tipo.pattern, tipo.color);
  }

// Mueve la nave
void mover_n (int x)
  {putimage (navex, disty, imagen_n, XOR_PUT),
   putimage (x, disty, imagen_n, XOR_PUT),
   navex = x;
  }

// Dibuja y almacena los datos de todos los enemigos
void crear_enemigos ()
  {int i, j, x, y = margen + 20, n = 0;
   for (i=0; i<7; i++)
     {x = 240;
      for (j=0; j<7; j++)
	putimage (x,y, imagen_e, XOR_PUT),
	enemigo [n].x = x, enemigo [n].y = y, enemigo [n].visible = 1, enemigo [n].enmovimiento = 0,
	n++,
	x += 20;
      y += 20;
     }
  }

// Mueve a los enemigos en la pantalla
void mover_e ()
  {int i;
   for (i=0; i<=maxenemigos; i++)
     {if (enemigo [i].x >= navex && enemigo [i].x <= navex + ancho && enemigo [i].y >= disty && enemigo [i].y<= disty + ancho / 2)
	{vidas--,
	 putimage (enemigo [i].x, enemigo [i].y, imagen_e, XOR_PUT),
	 enemigo [i].x = 0,
	 enemigo [i].y = 0,
	 enemigo [i].visible = 0,
	 enemigo [i].enmovimiento = 0,
	 destruidos++,
	 estallido_n (),
	 barra_estado ();
	}
      if (enemigo [i].visible && enemigo [i].enmovimiento)
	{putimage (enemigo [i].x, enemigo [i].y, imagen_e, XOR_PUT),
	 enemigo [i].x += random (10),
	 enemigo [i].y += random (10);
	 if (enemigo [i].y >= maxy) enemigo [i].y = margen;
	 if (enemigo [i].x >= maxx) enemigo [i].x = 0;
	 putimage (enemigo [i].x, enemigo [i].y, imagen_e, XOR_PUT);
	}
     }
  }

// Inicializa un disparo de la nave
void disparar ()
  {int color = getcolor ();
   setcolor (12),
   bala.x = navex + ancho / 2;
   if (!bala.visible)
     line (bala.x,bala.y1, bala.x, bala.y2),
     bala.visible = 1;
   setcolor (color);
  }

// Mueve a la bala inicializada
void avanzar ()
  {static int contador = 0;
   int i, seleccionado, color = getcolor ();
   setcolor (12),
   setwritemode (XOR_PUT);
   for (i=0; i<maxenemigos; i++)
     if (bala.visible && enemigo [i].visible)
	if (bala.x >= enemigo [i].x && bala.x <= enemigo [i].x + ancho_e && bala.y1 >= enemigo [i].y && bala.y1 <= enemigo [i].y + ancho_e)
	  {line (bala.x, bala.y1, bala.x, bala.y2),
	   putimage (enemigo [i].x, enemigo [i].y, imagen_e, XOR_PUT),
	   putimage (enemigo [i].x, enemigo [i].y, explosion, XOR_PUT),
	   estallido_e (),
	   putimage (enemigo [i].x, enemigo [i].y, explosion, XOR_PUT),
	   enemigo [i].visible = 0,
	   destruidos++,
	   barra_estado ();
	   if (enemigo [i].enmovimiento)
	     {enemigo [i].enmovimiento = 0;
	      do
		seleccionado = random (maxenemigos),
		enemigo [seleccionado].enmovimiento = 1;
	      while (!enemigo [seleccionado].visible && destruidos != maxenemigos);
	     }
	   bala.y1 = disty - 6,
	   bala.y2 = disty - 1,
	   bala.visible = 0;
	  }
   if (bala.visible && !(contador % retardo_tiro))
     contador = 0,
     line (bala.x, bala.y1, bala.x, bala.y2),
     bala.y1--,
     bala.y2--,
     line (bala.x, bala.y1, bala.x, bala.y2);
   if (bala.y2 < 0)
     bala.y1 = disty - 6,
     bala.y2 = disty - 1,
     bala.visible = 0;
   contador++,
   setwritemode (COPY_PUT),
   setcolor (color);
  }

// Escribe el numero de vidas y la cantidad de enemigos en pantalla
void barra_estado ()
  {int color = getcolor ();
   struct fillsettingstype tipo;
   setfillstyle (1, 8),
   settextjustify (CENTER_TEXT, CENTER_TEXT),
   bar (0,0, maxx,margen),
   setcolor (WHITE),
   escribe (medx, margen/2, "Nivel = %d  Vidas = %d  Meteoros destruidos = %d  Meteoros restantes = %d", nivel, vidas, destruidos, maxenemigos-destruidos);
   setcolor (color),
   setfillstyle (tipo.pattern, tipo.color);
   settextjustify (LEFT_TEXT, TOP_TEXT);
  }

// Produce el sonido de la explosion de un enemigo
void estallido_e ()
  {sound (400),
   delay (30),
   nosound (),
   sound (800),
   delay (30),
   nosound ();
  }

// Produce el sonido de la destruccion de la nave
void estallido_n ()
  {int i;
   for (i=3; i<45; i++)
     sound (400 + (i*5)),
     delay(10);
   nosound ();
  }

// Cierra graficos, libera memoria y termina
void terminar ()
  {settextjustify (CENTER_TEXT, CENTER_TEXT),
   setcolor (14);
   (destruidos == maxenemigos && vidas)? escribe (medx, medy-20, "GANASTE!!!") : escribe (medx, medy-20, "PERDISTE!!!");
   escribe (medx, medy, "JUEGO TERMINADO"),
   escribe (medx, medy+20, "Presiona una tecla"),
   getch (),
   free (imagen_n),
   free (imagen_e),
   closegraph (),
   exit (0);
  }

int juego ()
  {int i, ax = 0, velocimetro=0;
   mouse_data mouse;
   for (i=0; i<500; i++)
     putpixel (random (maxx), random (maxy), random (7) + 1);
   putimage (navex, disty, imagen_n, XOR_PUT),
   crear_enemigos (),
   barra_estado (),
   init_mouse (),
   ocultar_mouse ();
   do
     {leer_mouse (&mouse);
      if (mouse.x != ax)
	ax = mouse.x,
	mover_n (mouse.x);
      if (cont < maxmov && !(velocimetro % retardo))
	enemigo [random (maxenemigos)].enmovimiento = 1;
      if (!(velocimetro % retardo))
	cont++,
	mover_e (),
	velocimetro = 0;
      if (presionar_boton (BOT_IZQ, &mouse) && !bala.visible)
	disparar ();
      avanzar ();
      if (destruidos == maxenemigos)
	{subir_nivel ();
	 return 0;
	}
      if (!vidas)
	terminar ();
      velocimetro++;
      if (kbhit ())
	return 1;
     }
   while (1);
  }

void subir_nivel ()
  {int color = getcolor ();
   nivel++;
   if (retardo > 10) retardo -= 10;
   if (retardo_tiro > 1) retardo_tiro--;
   cont = 0,
   destruidos = 0;
   if (maxmov <= maxenemigos) maxmov++;
   if (!(nivel % 5))
     vidas++;
   setcolor (14),
   settextjustify (CENTER_TEXT, CENTER_TEXT),
   escribe (medx, medy-20, "GANASTE!!!"),
   escribe (medx, medy, "Avanzas al nivel %d", nivel),
   escribe (medx, medy+20, "Presiona una tecla para continuar"),
   getch (),
   cleardevice (),
   setcolor (color);
  }